# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from string import Template
import pkg_resources


def process_sql_template(filename, mappings=None):
    """
    Reads a templated SQL file and substitutes any variables
    :param filename: the SQL file
    :param mappings: the variables to be substituted
    :return: the text of the SQL query with any variables substituted
    """
    # https://stackoverflow.com/questions/6028000/how-to-read-a-static-file-from-inside-a-python-package Option 1
    resource_package = 'data_warehouse_client'
    resource_path = '/'.join(('sql', filename))
    sql_template = pkg_resources.resource_string(resource_package, resource_path)
    sql_template_bytes = sql_template.decode('utf8')
    data = ' '.join(sql_template_bytes.replace('\r\n', ' ').split())
    res = Template(data).substitute(mappings)
    return res



# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Warehouse Checker code - check:
# valtype field points to non-NULL value field
# The measurementType is in the measurement group for each measurement
# The valtype is correct for the measurement type for each measurement
# the measurement group is valid for the study
# there are no missing measurement types in a measurement group instance
# no ordinal, nominal, bounded integer or bounded real values are out of bounds


from data_warehouse_client import file_utils
from data_warehouse_client import data_warehouse
from data_warehouse_client import print_io
from tabulate import tabulate
import datetime


def check_category_exists(dw, study):
    """
    Find measurement types of nominal or ordinal value type without entry in category table
    :param dw: handle to data warehouse
    :param study: study id
    :return: the ids and names of measurement types in the study that fail the test
    """
    mappings = {"study": str(study)}
    query = file_utils.process_sql_template("ordinal_types_not_matching_category.sql", mappings)
    return dw.return_query_result(query)


def check_integer_bounds_exist(dw, study):
    """
    Find measurement types of bounded integer value type without entry in boundsinteger table
    :param dw: handle to data warehouse
    :param study: study id
    :return: the ids and names of measurement types in the study that fail the test
    """
    mappings = {"study": str(study)}
    query = file_utils.process_sql_template("bounded_integer_measurement_types_without_bounds.sql", mappings)
    return dw.return_query_result(query)


def check_real_bounds_exist(dw, study):
    """
    Find measurement types of bounded real value type without entry in boundsreal table
    :param dw: handle to data warehouse
    :param study: study id
    :return: the ids and names of measurement types in the study that fail the test
    """
    mappings = {"study": str(study)}
    query = file_utils.process_sql_template("bounded_real_measurement_types_without_bounds.sql", mappings)
    return dw.return_query_result(query)


def check_valtype_matches_values(dw, study):
    """
    Find measurements that lack a value
    :param dw: handle to data warehouse
    :param study: study id
    :return: the measurements in the study that fail the test
    """
    mappings = {"study": str(study), "core_sql": data_warehouse.core_sql_for_measurements()}
    query = file_utils.process_sql_template("measurements_lacking_value.sql", mappings)
    return dw.return_query_result(query)


def check_category_in_range(dw, study):
    """
    Returns the ids of measurements that refer to a non-existent category
    :param dw: handle to data warehouse
    :param study: study id
    :return: the ids of measurements in the study that fail the test
    """
    mappings = {"study": str(study), "core_sql": data_warehouse.core_sql_for_measurements()}
    query = file_utils.process_sql_template("measurements_lacking_value.sql", mappings)
    return dw.return_query_result(query)


def check_bounded_integers(dw, study):
    """
    Returns the ids of measurements that hold bounded integers that are out of range
    :param dw: handle to data warehouse
    :param study: study id
    :return: the ids of measurements in the study that fail the test
    """
    mappings = {"study": str(study)}
    query = file_utils.process_sql_template("bounded_integers.sql", mappings)
    return dw.return_query_result(query)


def check_bounded_reals(dw, study):
    """
    Returns the ids of measurements that hold bounded reals that are out of range
    :param dw: handle to data warehouse
    :param study: study id
    :return: the ids of measurements in the study that fail the test
    """
    mappings = {"study": str(study)}
    query = file_utils.process_sql_template("bounded_reals.sql", mappings)
    return dw.return_query_result(query)


def print_check_warehouse(dw, study):
    """
    Prints on stdout the results of all the tests that check the warehouse for errors
    :param dw: handle to data warehouse
    :param study: study id
    :return:
    """
    print(f'Check Study {study}')
    print()

    print(f'- Check Metadata')
    print()

    print(f'-- Measurement types declared as ordinal or nominal but without entries in the Category Table')
    r2 = check_category_exists(dw, study)
    n_errors = len(r2)
    if n_errors > 0:
        print(tabulate(r2, headers=['Measurement Type Id', 'Message Type Name']))
    print(f'({n_errors} invalid entries)')

    print(f'-- Measurement types declared as bounded integer but without entries in the Boundsint Table')
    rbi = check_integer_bounds_exist(dw, study)
    n_errors = len(rbi)
    if n_errors > 0:
        print(tabulate(rbi, headers=['Measurement Type Id', 'Message Type Name']))
    print(f'({n_errors} invalid entries)')

    print(f'-- Measurement types declared as bounded real but without entries in the Boundsreal Table')
    rbr = check_real_bounds_exist(dw, study)
    n_errors = len(rbr)
    if n_errors > 0:
        print(tabulate(rbr, headers=['Measurement Type Id', 'Message Type Name']))
    print(f'({n_errors} invalid entries)')

    print(f'- Check Measurements')
    print()

    print(f'-- Measurements where the Value Type does not match the values stored in the Measurement Table')
    r1 = check_valtype_matches_values(dw, study)
    n_invalid_entries = len(r1)
    if n_invalid_entries > 0:
        print_io.print_measurements(r1)
    print(f'({n_invalid_entries} invalid entries)')

    print(f'-- Measurements declared as ordinal or nominal that refer to a non-existent category')
    r3 = check_category_in_range(dw, study)
    n_errors = len(r3)
    if n_errors > 0:
        print(tabulate(r3, headers=['Measurement Id']))
    print(f'({n_errors} measurements)')

    print()
    print(f'-- Measurements declared as Bounded Integers whose value is outside of the bounds')
    r4 = check_bounded_integers(dw, study)
    n_errors = len(r4)
    if n_errors > 0:
        print(tabulate(r4, headers=['Id', 'Value']))
    print(f'({n_errors} measurements)')

    print()
    print(f'-- Measurements declared as Bounded Reals whose value is outside of the bounds')
    r5 = check_bounded_reals(dw, study)
    n_errors = len(r5)
    if n_errors > 0:
        print(tabulate(r5, headers=['Id', 'Value']))
    print(f'({n_errors} measurements)')
    print()


def mk_txt_report_file_name(f_dir, report_name, time_string):
    return f_dir + report_name + time_string + ".txt"


def print_check_warehouse_to_file(dw, study):
    """
    Prints to a file the results of all the tests that check the warehouse for errors
    :param dw: handle to data warehouse
    :param study: study id
    :return:
    """

    file_dir = "reports/"
    timestamp = datetime.datetime.now()  # use the current date and time if none is specified
    time_fname_str = timestamp.strftime('%Y-%m-%dh%Hm%Ms%S')
    fname = mk_txt_report_file_name(file_dir, "warehouse-check", time_fname_str)

    with open(fname, "w") as f:
        print(f'Check Study {study}\n', file=f)

        print(f'- Check Metadata\n', file=f)

        print(f'-- Measurement types declared as ordinal or nominal but without entries in the Category Table', file=f)
        r2 = check_category_exists(dw, study)
        n_errors = len(r2)
        if n_errors > 0:
            print(tabulate(r2, headers=['Measurement Type Id', 'Message Type Name']))
        print(f'({n_errors} invalid entries)', file=f)

        print(f'-- Measurement types declared as bounded integer but without entries in the Boundsint Table', file=f)
        rbi = check_integer_bounds_exist(dw, study)
        n_errors = len(rbi)
        if n_errors > 0:
            print(tabulate(rbi, headers=['Measurement Type Id', 'Message Type Name']), file=f)
        print(f'({n_errors} invalid entries)', file=f)

        print(f'-- Measurement types declared as bounded real but without entries in the Boundsreal Table', file=f)
        rbr = check_real_bounds_exist(dw, study)
        n_errors = len(rbr)
        if n_errors > 0:
            print(tabulate(rbr, headers=['Measurement Type Id', 'Message Type Name']), file=f)
        print(f'({n_errors} invalid entries)', file=f)

        print(f'- Check Measurements\n', file=f)

        print(f'-- Measurements where the Value Type does not match the values stored in the Measurement Table', file=f)
        r1 = check_valtype_matches_values(dw, study)
        n_invalid_entries = len(r1)
        if n_invalid_entries > 0:
            print_io.print_measurements_to_file(r1, f)
        print(f'({n_invalid_entries} invalid entries)', file=f)

        print(f'-- Measurements declared as ordinal or nominal that refer to a non-existent category', file=f)
        r3 = check_category_in_range(dw, study)
        n_errors = len(r3)
        if n_errors > 0:
            print(tabulate(r3, headers=['Measurement Id']), file=f)
        print(f'({n_errors} measurements)', file=f)

        print()
        print(f'-- Measurements declared as Bounded Integers whose value is outside of the bounds', file=f)
        r4 = check_bounded_integers(dw, study)
        n_errors = len(r4)
        if n_errors > 0:
            print(tabulate(r4, headers=['Id', 'Value']), file=f)
        print(f'({n_errors} measurements)\n', file=f)

        print(f'-- Measurements declared as Bounded Reals whose value is outside of the bounds', file=f)
        r5 = check_bounded_reals(dw, study)
        n_errors = len(r5)
        if n_errors > 0:
            print(tabulate(r5, headers=['Id', 'Value']), file=f)
        print(f'({n_errors} measurements)\n', file=f)



# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

def form_measurements(rows):
    """
    The raw query results within getMeasurements, getMeasurementsWithValueTest and
        contain a column for each possible type of value:
            integer, real, datatime, string. Each is set to null apart from the one that holds the value appropriate
            for the type of measurement. This function replaces those columns with a single field holding the actual
            value
    :param rows: list of rows returned by a query
    :return: list of rows, each representing one measurement in a list with elements:
            id,time,study,participant,measurementType,typeName,measurementGroup, groupInstance,trial,valType,value
    """
    n_rows: int = len(rows)
    n_cols: int = 11
    row_out = [[None] * n_cols for i in range(n_rows)]
    val_type_index = 9
    value_index = 10
    for r in range(n_rows):
        # Indices: 0 = id, 1 = time, 2 = study, 3 = participant, 4 = measurementType, 5 = measurementTypeName,
        # 6 = measurementGroup, 7 = measurementGroupInstance, 8 = trial, 9 = valType, 10 = value
        for x in range(value_index):
            row_out[r][x] = rows[r][x]
        if rows[r][val_type_index] == 0:  # integer
            row_out[r][value_index] = rows[r][10]
        elif rows[r][val_type_index] == 1:  # real
            row_out[r][value_index] = rows[r][11]
        elif rows[r][val_type_index] == 2:  # text
            row_out[r][value_index] = rows[r][12]
        elif rows[r][val_type_index] == 3:  # datetime
            row_out[r][value_index] = rows[r][13]
        elif rows[r][val_type_index] == 4 and rows[r][value_index] == 0:  # boolean False
            row_out[r][value_index] = "F"
        elif rows[r][val_type_index] == 4 and rows[r][value_index] == 1:  # boolean True
            row_out[r][value_index] = "T"
        elif rows[r][val_type_index] == 5:  # nominal
            row_out[r][value_index] = rows[r][14]
        elif rows[r][val_type_index] == 6:  # ordinal
            row_out[r][value_index] = rows[r][14]
        elif rows[r][val_type_index] == 7:  # boundedint
            row_out[r][value_index] = rows[r][10]
        elif rows[r][val_type_index] == 8:  # boundedreal
            row_out[r][value_index] = rows[r][11]
        else:
            print("typeval error of ", rows[r][9], " for id", rows[r][0], " study ", rows[r][2])
    return row_out


def form_measurement_group(dw, study, measurement_group, rows):
    """
    Creates a result where each measurement group instance occupies one row.
    :param dw: data warehouse handle
    :param study: study id
    :param measurement_group: measurement group id
    :param rows: list of rows returned by getMeasurementGroupInstancesWithValueTests or
                    getMeasurements (where it returns whole measurement group instances - i.e. where
                                    measurementType is not specified, but measurement group or
                                    measurement group instance is specified)
    :return: (header, list of rows, each representing one measurement group instance, held in a list with elements:
            groupInstance,time of first measurement in instance,study,participant,measurementGroup,trial,
                value1, value2....
            where value n is the value for the nth measurement in the instance (ordered by measurement type)
            "None" is used if a value is missing)
    """
    result_rows = []
    if len(rows) > 0:
        measurement_group: int = rows[0][6]   # get the measurement group from the first measurement
        mts = dw.get_type_ids_in_measurement_group(study, measurement_group)  # get all the measurement type ids
        # Create a dictionary entry for each instance hoding the common values and the measurement values
        result_values = {}  # the measurement values
        result_common = {}  # the common values returned for each instance: instance, time of 1st measurement,
                            #                                               study, participant, measurementGroup, trial
        for (row_id, time, study_id, participant, mt, tn, mg, mgi, trial, val_type, value) in rows:
            if not (mgi in result_common):           # it's a new instance
                result_common.update({mgi: [mgi, time, study, participant, mg, trial]})  # store the common values
                result_values.update({mgi: {}})                                          # create empty values for inst
            result_values[mgi][mt] = value  # add values to the dictionary               # add the value
        # Write out the list of measurement groups
        for instance in result_values:               # for each instance
            val_dict = result_values[instance]       # get the values for that instance
            values = []
            for mt in mts:                           # for each measurement type in the measurement group
                val = val_dict.get(mt, None)         # get the value - the default is None if no value exists
                values = values + [val]              # add the value to the list of values
            result_rows = result_rows + [(result_common[instance] + values)]  # add entry for instance to the result

    # prepare the header
    type_names = dw.get_types_in_a_measurement_group(study, measurement_group)
    n_types: int = len(type_names)
    header = ["Instance", "Time", "Study", "Participant", "Measurement Group", "Trial"]
    for t in range(n_types):
        header.append(type_names[t][0])
    return header, result_rows



# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import List
from tabulate import tabulate


def print_rows(rows, header: List[str]):
    """
    prints each row returned by a query
    :param rows: a list of rows. Each row is a list of fields
    :param header: a list of field names
    """
    print(tabulate(rows, headers=header))


def print_measurement_group_instances(header, instances):
    """
    Prints a list of measurement group instances, converting the datetime to strings
    The output file has a row for each measurement group instance. The fields are:
        groupInstance,time of first measurement in instance,study,participant,measurementGroup,trial,
                value1, value2....
            where value n is the value for the nth measurement in the instance (ordered by measurement type)
    :param header: a list of column names
    :param instances: a list of measurement group instances in the format produced by formatMeasurementGroup
    """
    print(tabulate(instances, headers=header))


def print_measurements(rows):
    """
    Prints a list of measurements, converting the datetimes to strings
    :param rows: a list of measurements with the elements id,time,study,participant,measurementType,
                    typeName,measurementGroup,groupInstance,trial,valType,value
    """
    header_row: List[str] = ["Id", "Time", "Study", "Participant", "MeasurementType", "Type Name",
                             "Measurement Group", "Group Instance", "Trial", "Val Type", "Value"]
    print(tabulate(rows, headers=header_row))


def print_rows_to_file(rows, header: List[str], f_handle):
    """
    prints to a file each row returned by a query
    :param rows: a list of rows. Each row is a list of fields
    :param header: a list of field names
    :param f_handle: an open file handle
    """
    print(tabulate(rows, headers=header), file=f_handle)


def print_measurement_group_instances_to_file(header, instances, f_handle):
    """
    Prints to a file a list of measurement group instances, converting the datetime to strings
    The output file has a row for each measurement group instance. The fields are:
        groupInstance,time of first measurement in instance,study,participant,measurementGroup,trial,
                value1, value2....
            where value n is the value for the nth measurement in the instance (ordered by measurement type)
    :param header: a list of column names
    :param instances: a list of measurement group instances in the format produced by formatMeasurementGroup
    :param f_handle: an open file handle
    """
    print(tabulate(instances, headers=header), file=f_handle)


def print_measurements_to_file(rows, f_handle):
    """
    Prints to a file  a list of measurements, converting the datetimes to strings
    :param rows: a list of measurements with the elements id,time,study,participant,measurementType,
                    typeName,measurementGroup,groupInstance,trial,valType,value
    :param f_handle: an open file handle
    """
    header_row: List[str] = ["Id", "Time", "Study", "Participant", "MeasurementType", "Type Name",
                             "Measurement Group", "Group Instance", "Trial", "Val Type", "Value"]
    print(tabulate(rows, headers=header_row), file=f_handle)



# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import List
import csv


def export_measurements_as_csv(rows, fname):
    """
    Stores measurements returned by queries in a CSV file
    The input rows must be in the format produced by:
        getMeasurements or getMeasurementsWithValueTest
        The output file has a header row, followed by a row for each measurement. This has the columns:
            id,time,study,participant,measurementType,typeName,measurementGroup, groupInstance,trial,valType,value
    :param rows: a list of rows returned by getMeasurements, getMeasurementsWithValueTest or
                    getMeasurementGroupInstancesWithValueTests
    :param fname: the filename of the output CSV file
    """
    with open(fname, "w", newline="") as f:
        writer = csv.writer(f)
        writer.writerow(
            ["Id", "Time", "Study", "Participant", "Measurement Type", "Measurement Type Name", "Measurement Group",
             "Measurement Group Instance", "Trial", "Value Type", "Value"])
        writer.writerows(rows)


def export_measurement_groups_as_csv(header, instances, fname):
    """
    Stores measurements returned by formMeasurementGroups in a CSV file
    The input rows must be in the format produced by formMeasurementGroups
    The output file has a header row, followed by a row for each measurement group instance. The table has columns:
        groupInstance,time of first measurement in instance,study,participant,measurementGroup,trial,
                value1, value2....
            where value n is the value for the nth measurement in the instance (ordered by measurement type)
    :param header: a list of column names
    :param instances: a list of instances returned by formatMeasurementGroup
    :param fname: the filename of the output CSV file
    """
    with open(fname, "w", newline="") as f:
        writer = csv.writer(f)
        writer.writerow(header)
        writer.writerows(instances)



# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from data_warehouse_client import data_warehouse
from data_warehouse_client import print_io
from data_warehouse_client import study_summary
from data_warehouse_client import plot
from data_warehouse_client import csv_io
import datetime


# Create a connection to the data warehouse
dw = data_warehouse.DataWarehouse("db-credentials-read-only.json", "datawarehouse")

# Sample queries...
print("\nQ1: All measurements in measurement group 6 in study 4\n")
q1res = dw.get_measurements(4, measurement_group=6)
print_io.print_measurements(q1res)

print("\nQ2: All measurements for participant 1 in study 4\n")
q2res = dw.get_measurements(4, participant=1)
print_io.print_measurements(q2res)

print("\nQ3: All the measurements of type 1 in study 4\n")
q3res = dw.get_measurements(4, measurement_type=220)
print_io.print_measurements(q3res)

print("\nQ4: All measurements in Measurement Group 1, in Study 1\n")
(header, instances) = dw.get_measurement_group_instances(1, 1, [])
print_io.print_measurement_group_instances(header, instances)

print("\nQ5: All results for participant 1 in study 4\n")
resultRows = dw.get_measurements(4, participant=1)
print_io.print_measurements(resultRows)

print("\nQ6: All measurementgroups and types in study 1\n")
print_io.print_rows(dw.get_all_measurement_groups_and_types_in_a_study(4),
                    ["Measurement Group", "Measurement Type", "Type Name"])

print("\nQ7: All results in measurement group 16 for study 3\n")
mgis = dw.get_measurements(3, measurement_group=16)
print_io.print_measurements(mgis)

print("\nQ8: Plot all results of type 155 in study 3\n")
mts = dw.get_measurements(3, measurement_type=155)  # these are continuous glucose monitor measurements
plot.plot_measurements(dw, mts, 3, 155, 'output/example155.png')

print("\nQ9: Create a CSV file for all results of type 155 in study 3\n")
csv_io.export_measurements_as_csv(mts, 'output/example155.csv')

print("\nQ10: All measurements with type = 152 in study 4\n")
ms2 = dw.get_measurements(4, measurement_type=152)
print_io.print_measurements(ms2)
plot.plot_measurements(dw, ms2, 4, 152, 'output/example2.png')

print("\nQ11: All measurements with measurementgroup = 15 in study 4\n")
(header, instances) = dw.get_measurement_group_instances(4, 15, [])
print_io.print_measurement_group_instances(header, instances)

print("\nQ12: Plot all measurements in study 3 with type = 155 where the value is greater that 9\n")
res = dw.get_measurements_with_value_test(3, 155, "> 9")
print_io.print_measurements(res)

print("\nQ13: Create a CSV file for all measurements in study 3 with type = 155 where the value is greater that 9\n")
csv_io.export_measurements_as_csv(res, 'output/example2.csv')

print("\nQ14: All measurements in group 15 in study 4")
(header, instances) = dw.get_measurement_group_instances(4, 15, [])
print_io.print_measurement_group_instances(header, instances)

print("\nQ15: All instances of measurement group 15 where the participant's age is greater than 22")
print("          and body mass is less than 55kgs in study 2")
(header, instances) = dw.get_measurement_group_instances(2, 15, [(151, ">22"), (154, "<55.0")])
print_io.print_measurement_group_instances(header, instances)

print("\nParticipants: ")
parts = data_warehouse.get_participants_in_result(instances)
print(*parts, sep=',')
print()

(header, instances) = dw.get_measurement_group_instances_for_cohort(2, 15, parts, [])
print_io.print_measurement_group_instances(header, instances)

(header, instances) = dw.get_measurement_group_instances_for_cohort(15, 2, [4], [])
print_io.print_measurement_group_instances(header, instances)

print("\nQ16: The output of Q15 stored as a CSV\n")
csv_io.export_measurement_groups_as_csv(header, instances, 'output/example17.csv')

print("\nQ17: The average of all measurements of type 155 from study 3\n")
ms7 = dw.aggregate_measurements(3, 155, "avg")
print(ms7)

print("\nQ18: All measurements for Study 5\n")
ms18 = dw.get_measurements(study=5)
print_io.print_measurements(ms18)

print("\nQ19: All measurements in group 20 for Study 5 measurement group 20 in tabular form\n")
(header, instances) = dw.get_measurement_group_instances(5, 20, [])
print_io.print_measurement_group_instances(header, instances)

print("\n   : store in CSV file\n")
csv_io.export_measurement_groups_as_csv(header, instances, 'output/example19.csv')

print("\nQ20: All measurements for Study 5, trial 2, measurement group 20 in tabular form\n")
(header, instances) = dw.get_measurement_group_instances(5, 20, [], trial=2)
print_io.print_measurement_group_instances(header, instances)

print("\nQ21: All measurements for Study 5, measurement group 20, trial 4 in tabular form\n")
(header, instances) = dw.get_measurement_group_instances(5, 20, [], trial=4)
print_io.print_measurement_group_instances(header, instances)

print("\nQ22: All measurements for Study 5, measurement Group 20, from 2018\n")
year_query = "SELECT measurement.id,measurement.time,measurement.measurementtype,measurement.valreal" +\
             " FROM  measurement " + \
             " WHERE  EXTRACT(YEAR FROM measurement.time) = '2018' AND " +\
             "        measurement.measurementgroup = 20;"

ms22 = dw.return_query_result(year_query)
ms22Header = ["Id", "Time", "Measurement Type", "Value"]
print_io.print_rows(ms22, ms22Header)

print("\nQ23: Cohort-based analysis: all measurements from all participants with Multiple Sclerosis in study 5")
ms23 = dw.get_measurements_by_cohort(5, 2)
print_io.print_measurements(ms23)

print("\nQ26: All measurements in group 6 for Study 27")
(header, instances) = dw.get_measurement_group_instances(27, 6, [])
print_io.print_measurement_group_instances(header, instances)
print("\n")

print("\nQ27: All measurements in Study 5")
study_summary.print_all_instances_in_a_study(dw, 5)

print("\nQ28: All measurement group 14 measurements for all participants in UNEW and USFD with HA or CHF in study 26")
# Retrieve the cohort
# cohort = mobilise_cohort_selection.get_mobilise_cohort(data_warehouse, 26, ["UNEW","USFD"], ["HA","CHF"])
# Use the cohort in a query
# print(*cohort, sep=',')
# ms28 = data_warehouse.get_measurement_group_instances_for_cohort(14, 26, cohort, [])
# mgs28 = data_warehouse.formMeasurementGroup(14, ms28)
# data_warehouse.printMeasurementGroupInstances(mgs28, 14, 26)

print("\nQ29: All instances of measurement group 15 where the participant's age is greater than 22")
print("          and body mass is less than 55kgs in study 2")
(header, instances) = dw.get_measurement_group_instances(2, 15, [(151, ">22"), (154, "<55.0")])
# data_warehouse.printMeasurements(ms6)
print_io.print_measurement_group_instances(header, instances)

# print()
# print("Run Warehouse Checker\n")
# warehouse_checker.print_check_warhouse(dw, 4)

print("\nQ30: All instances of the x_accelerometer in measurement group 85 between 10:00 and 11:00 on 16-6-2020")
ms = dw.get_measurements(85, measurement_group=0, measurement_type=2,
                         start_time=datetime.datetime(2009, 6, 16, hour=10),
                         end_time=datetime.datetime(2009, 6, 16, hour=11))
plot.plot_measurements(dw, ms, 85, 2, 'output/example85.png')
# study_to_save = 4
# file_name = "c:/Temp/study4metadata.json"
# print(f'Save Metadata from Study {study_to_save}')
# table_writer_json.data_warehouse_metadata_tables_to_file(dw, study_to_save, file_name)



# Copyright 2020 Newcastle University.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from data_warehouse_client import table_writer_json


def clone_study_metadata(dw, source_study_id, dest_study_id):
    """
    Copies all the metadata from one study to another
    :param dw: data warehouse end point
    :param source_study_id: original study
    :param dest_study_id: destination study
    :return total rows inserted
    """
    # extract existing metadata
    tbls = table_writer_json.data_warehouse_metadata_tables_to_dictionary(dw, source_study_id)
    for table in tbls:  # for each table
        for row in tbls[table]:  # for each row
            row.update({'study': dest_study_id})  # update all occurrences of study to the destination study
    return table_writer_json.write_tables_in_dw_from_dictionary(dw, tbls)  # write the result to the warehouse
